/***********************************************************
 * $Id: ReturnCallbackHandler.java 68 2009-05-02 21:23:05Z wglas $
 * 
 * remoting framework of the clazzes project.
 * http://www.clazzes.org
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Created: 29.04.2009
 * 
 ***********************************************************/

package org.clazzes.remoting.beans;

import java.rmi.server.UID;

import org.clazzes.remoting.InvocationHandler;

/**
 * A handler for calls to callback proxies returned by the server to the client.
 * 
 * @author wglas
 */
public interface ReturnCallbackHandler extends InvocationHandler {

	/**
	 * The property under which the GUID of the server-side proxy is stored in the invocation's
	 * request payload.
	 * 
	 * @see InvocationRequest#getRequestPayload()
	 */
	public static final String RETURN_PROXY_GUID_PROPERTY = "org.clazzes.remoting.returnProxy";
	
	/**
	 * @param iface The interface of a return value.
	 * @return Whether this interface is registered as a callback interface.
	 */
	public boolean isCallbackInterface(Class<?> iface);
	
	/**
	 * @param sessionId A client session Id.
	 * @param iface The interface of a service return value.
	 * @param proxy The proxy object to be served.
	 * @return A UID for the registered proxy return value.
	 */
	UID registerCallback(UID sessionId, Class<?> iface, Object proxy);
	
	/**
	 * Deregister a single callback of a given client.
	 * 
	 * @param sessionId A client session Id.
	 * @param guid The GUID of the callback to be deregistered a returned by
	 *             {@link #registerCallback(String, Class, Object)}.
	 */
	void deregisterCallback(UID sessionId, UID guid);
	
	/**
	 * Deregister all callbacks of a given client.
	 * 
	 * @param sessionId A client session Id.
	 */
	void deregisterAllCallbacks(UID sessionId);
	
    /**
     * Count all callbacks of a given client.
     * 
     * @param sessionId A client session Id.
     * @return The number of callbacks currently registered for the given client ID.
     */
	int countCallbacks(UID sessionId);
}
