/***********************************************************
 * $Id: SessionContextInvocationHandler.java 102 2009-08-09 18:55:15Z wglas $
 * 
 * remoting framework of the clazzes project.
 * http://www.clazzes.org
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Created: 29.04.2009
 * 
 ***********************************************************/

package org.clazzes.remoting.beans;

import java.rmi.server.UID;
import java.util.Map;

import org.clazzes.remoting.InvocationContext;
import org.clazzes.remoting.InvocationHandler;
import org.clazzes.remoting.server.ClientTerminationListener;

/**
 * An invocation handler, which manages the hibernate session attached to the current
 * thread.
 * 
 * @author wglas
 */
public class SessionContextInvocationHandler implements InvocationHandler, ClientTerminationListener {

	//private static final Log log = LogFactory.getLog(SessionContextInvocationHandler.class);

	private SessionManager sessionManager;
	private ReturnCallbackHandler returnCallbackHandler;
	
	/* (non-Javadoc)
	 * @see org.clazzes.remoting.InvocationHandler#invoke(org.clazzes.remoting.InvocationContext, java.lang.Object, java.util.Map)
	 */
	public Object invoke(InvocationContext ctx, Object arg,
			Map<String, Object> metadata) throws Throwable {
	
		if (SessionCmd.OPEN_SESSION.equals(arg)) {
			
			if (this.sessionManager != null)
				this.sessionManager.openSession(ctx.getClientUID(),metadata);
			
			return SessionCmd.OPEN_SESSION;
		}
		else if (SessionCmd.CLOSE_SESSION.equals(arg)) {
			
			if (this.sessionManager != null) {
				
				this.sessionManager.closeSession(ctx.getClientUID());
			}
				
			if (this.returnCallbackHandler != null) {
				
				this.returnCallbackHandler.deregisterAllCallbacks(ctx.getClientUID());
			}
			
	        return SessionCmd.CLOSE_SESSION;
		}
		
		throw new UnsupportedOperationException("Operation ["+arg+"] is not supported.");
	}

	/* (non-Javadoc)
	 * @see org.clazzes.remoting.server.ClientTerminationListener#clientTerminated(java.rmi.server.UID)
	 */
	public void clientTerminated(UID clientUID) {
		
		if (this.sessionManager != null) {
			
			this.sessionManager.closeSession(clientUID);
		}
			
		if (this.returnCallbackHandler != null) {
			
			this.returnCallbackHandler.deregisterAllCallbacks(clientUID);
		}
	
	}

	/**
	 * @return the returnCallbackHandler
	 */
	public ReturnCallbackHandler getReturnCallbackHandler() {
		return this.returnCallbackHandler;
	}

	/**
	 * @param returnCallbackHandler the returnCallbackHandler to set
	 */
	public void setReturnCallbackHandler(ReturnCallbackHandler returnCallbackHandler) {
		this.returnCallbackHandler = returnCallbackHandler;
	}

	/**
	 * @return the sessionManager
	 */
	public SessionManager getSessionManager() {
		return this.sessionManager;
	}

	/**
	 * @param sessionManager the sessionManager to set
	 */
	public void setSessionManager(SessionManager sessionManager) {
		this.sessionManager = sessionManager;
	}
}
