/***********************************************************
 * $Id: SessionManager.java 66 2009-04-30 13:47:14Z wglas $
 * 
 * remoting framework of the clazzes project.
 * http://www.clazzes.org
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Created: 29.04.2009
 * 
 ***********************************************************/

package org.clazzes.remoting.beans;

import java.rmi.server.UID;
import java.util.Map;


/**
 * An implementation of this class maintains a server-side hibernate session,
 * which are associated to client session IDs.
 * 
 * @author wglas
 */
public interface SessionManager {

	/**
	 * Open a session and attach it to the current thread.
	 * 
	 * @param sessionId The client sessionId for which to open a session.
	 * @param metadata The request metadata as generated by {@link SessionAuthenticator}
	 *                 or by 
     * @throws SecurityException Upon authentication errors.
	 */
	public void openSession(UID sessionId, Map<String,Object> metadata) throws SecurityException;
	
	/**
	 * Close a session and withdraw it from the current thread.
	 * 
	 * @param sessionId The client sessionId of which to close a session previously
	 *                  created by {@link #openSession(String)}.
	 */
	public void closeSession(UID sessionId);
}
