/***********************************************************
 * $Id: ReturnClassBytesCmd.java 7 2008-10-16 20:35:10Z wglas $
 * 
 * remoting framework of the clazzes project.
 * http://www.clazzes.org
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Created: 15.10.2008
 * 
 ***********************************************************/

package org.clazzes.remoting.cmd;

import org.clazzes.remoting.loading.ClassBytes;


/**
 * A command that reports a remote class to the client.
 * 
 * @author wglas
 */
public class ReturnClassBytesCmd extends Cmd {

    private static final long serialVersionUID = 2641561482451151631L;

    private final ClassBytes classBytes;
    
    /**
     * @param classBytes The binary class being transported to the client.
     *              If null, the remote class could not be found.
     */
    public ReturnClassBytesCmd(ClassBytes classBytes) {
        
        super();
        this.classBytes = classBytes;
    }

    /**
     * @return The subsystem being called.
     */
    public ClassBytes getClassBytes() {
        return this.classBytes;
    }

    /* (non-Javadoc)
     * @see org.clazzes.remoting.cmd.Cmd#accept(org.clazzes.remoting.cmd.CmdVisitor)
     */
    @Override
    public void accept(CmdVisitor visitor) {
       
        visitor.visit(this);
    }
}
