/***********************************************************
 * $Id: ClassObjectMarshaler.java 144 2009-10-12 07:47:02Z wglas $
 * 
 * remoting framework of the clazzes project.
 * http://www.clazzes.org
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Created: 23.08.2009
 * 
 ***********************************************************/

package org.clazzes.remoting.marshal.impl;

import java.io.IOException;
import java.lang.reflect.Method;

/**
 * This marshaler reads/write a class by simply reading/writing the class name.
 *
 */
class ClassObjectMarshaler implements ObjectMarshaler {

    /* (non-Javadoc)
     * @see org.clazzes.remoting.marshal.impl.ObjectMarshaler#readObject(org.clazzes.remoting.marshal.impl.CompactMarshaler, long, org.clazzes.remoting.marshal.impl.ClassInfo)
     */
    public Object readObject(CompactMarshaler context, long ref, ClassInfo ci)
            throws IllegalArgumentException, IllegalAccessException,
            IOException, ClassNotFoundException {
      
        int tag = FieldMarshaler.readTag(context);
        
        if (FieldMarshaler.STRING_STREAM_ID != tag)
            throw new IllegalArgumentException("Invalid tag ["+tag+
                    "] found when reading java.ulang.Class className."); 
       
        String className = context.getDataInputStream().readUTF();
 
        Class<?> c = Class.forName(className,true,context.getClassLoader());
        
        context.cacheReadPremature(ref,c);
        return c;
    }

    /* (non-Javadoc)
     * @see org.clazzes.remoting.marshal.impl.ObjectMarshaler#writeObject(org.clazzes.remoting.marshal.impl.CompactMarshaler, java.lang.Object)
     */
    public void writeObject(CompactMarshaler context, Object obj)
            throws IllegalArgumentException, IOException,
            IllegalAccessException, ClassNotFoundException {
        
        Class<?> c = (Class<?>)obj;
        
        context.getDataOutputStream().write(FieldMarshaler.STRING_STREAM_ID);
        context.getDataOutputStream().writeUTF(c.getName());
    }

    public Method getReadResolveMethod() {
        return null;
    }

    public Method getWriteReplaceMethod() {
        return null;
    }

}
