/***********************************************************
 * $Id: EnumObjectMarshaler.java 190 2010-09-05 18:48:31Z remoting $
 * 
 * remoting framework of the clazzes project.
 * http://www.clazzes.org
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Created: 23.08.2009
 * 
 ***********************************************************/

package org.clazzes.remoting.marshal.impl;

import java.io.IOException;
import java.lang.reflect.Method;

/**
 * This marshaler reads/write an enum by simply reading/writing the name of an enum.
 */
class EnumObjectMarshaler implements ObjectMarshaler {

    
    public static final EnumObjectMarshaler INSTANCE = new EnumObjectMarshaler();
    
    /* (non-Javadoc)
     * @see org.clazzes.remoting.marshal.impl.ObjectMarshaler#readObject(org.clazzes.remoting.marshal.impl.CompactMarshaler, long, org.clazzes.remoting.marshal.impl.ClassInfo)
     */
    public Object readObject(CompactMarshaler context, long ref, ClassInfo ci)
            throws IllegalArgumentException, IllegalAccessException,
            IOException, ClassNotFoundException {
       
        int tag = FieldMarshaler.readTag(context);
        
        if (FieldMarshaler.INT_STREAM_ID != tag)
            throw new IllegalArgumentException("Invalid tag ["+tag+
                    "] found when reading an enum value of type ["+ci.getClazz()+"]."); 
       
        int ordinal = context.getDataInputStream().readInt();
        
        Class<?> c =  Class.forName(ci.getClazz(),true,context.getClassLoader());
        Object[] ec = c.getEnumConstants();
        
        Object e;
        
        if (ordinal < 0 || ordinal >= ec.length)
            e = null;
        else
            e = ec[ordinal];
        
        context.cacheReadPremature(ref,e);
        
        return e;
    }

    /* (non-Javadoc)
     * @see org.clazzes.remoting.marshal.impl.ObjectMarshaler#writeObject(org.clazzes.remoting.marshal.impl.CompactMarshaler, java.lang.Object)
     */
    public void writeObject(CompactMarshaler context, Object obj)
            throws IllegalArgumentException, IOException,
            IllegalAccessException, ClassNotFoundException {
        
        Enum<?> e = (Enum<?>) obj;
        
        context.getDataOutputStream().write(FieldMarshaler.INT_STREAM_ID);
        context.getDataOutputStream().writeInt(e.ordinal());
    }

    public Method getReadResolveMethod() {
        return null;
    }

    public Method getWriteReplaceMethod() {
        return null;
    }

}
